@@ -4,7 +4,7 @@ use super::super::codegen::{EnumVariation, CONSTIFIED_ENUM_MODULE_REPR_NAME};
 use super::analysis::{HasVtable, HasVtableResult, Sizedness, SizednessResult};
 use super::annotations::Annotations;
 use super::comment;
-use super::comp::MethodKind;
+use super::comp::{CompKind, MethodKind};
 use super::context::{BindgenContext, ItemId, PartialType, TypeId};
 use super::derive::{
     CanDeriveCopy, CanDeriveDebug, CanDeriveDefault, CanDeriveEq,
@@ -811,17 +811,23 @@ impl Item {
     /// Get this item's original C++ name, including any containing types, but without
     /// the namespace name. For nested types, the C++ name differs from the Rust name, e.g.
     /// a nested C++ type `A::B` would correspond to the Rust type `A_B`.
-    pub fn original_name(
-      &self,
-      ctx: &BindgenContext,
-    ) -> String {
+    /// If the item or any of its containing types is anonymous, returns None.
+    pub fn original_name(&self, ctx: &BindgenContext) -> Option<String> {
         let target = ctx.resolve_item(self.name_target(ctx));
 
-        // Concatenate name of item and all ancestors until we reach a namespace.
-        let mut names: Vec<_> = target
+        // Get item and all ancestors until the first enclosing namespace.
+        let ancestors: Vec<_> = target
             .ancestors(ctx)
             .map(|id| ctx.resolve_item(id))
             .take_while(|item| !item.is_module())
+            .collect();
+
+        if ancestors.iter().any(|item| item.is_anon()) {
+            return None;
+        }
+
+        let mut names: Vec<_> = ancestors
+            .iter()
             .map(|item| {
                 let target = ctx.resolve_item(item.name_target(ctx));
                 target.base_name(ctx)
@@ -831,7 +837,7 @@ impl Item {
 
         names.reverse();
 
-        names.join("::")
+        Some(names.join("::"))
     }
 
     /// Get the canonical name without taking into account the replaces
@@ -930,6 +936,12 @@ impl Item {
             names.push(base_name);
         }
 
+        if ctx.options().c_naming {
+            if let Some(prefix) = self.c_naming_prefix() {
+                names.insert(0, prefix.to_string());
+            }
+        }
+
         let name = names.join("_");
 
         let name = if opt.user_mangled == UserMangled::Yes {
@@ -1080,6 +1092,23 @@ impl Item {
         path.reverse();
         path
     }
+
+    /// Returns a prefix for the canonical name when C naming is enabled.
+    fn c_naming_prefix(&self) -> Option<&str> {
+        let ty = match self.kind {
+            ItemKind::Type(ref ty) => ty,
+            _ => return None,
+        };
+
+        Some(match ty.kind() {
+            TypeKind::Comp(ref ci) => match ci.kind() {
+                CompKind::Struct => "struct",
+                CompKind::Union => "union",
+            },
+            TypeKind::Enum(..) => "enum",
+            _ => return None,
+        })
+    }
 }
 
 impl<T> IsOpaque for T
@@ -1584,6 +1613,18 @@ impl ClangItemParser for Item {
             }
         }
 
+        // Treat all types that are declared inside functions as opaque. The Rust binding
+        // won't be able to do anything with them anyway.
+        //
+        // (If we don't do this check here, we can have subtle logic bugs because we generally
+        // ignore function bodies. See issue #2036.)
+        if let Some(ref parent) = ty.declaration().fallible_semantic_parent() {
+            if FunctionKind::from_cursor(parent).is_some() {
+                debug!("Skipping type declared inside function: {:?}", ty);
+                return Ok(Item::new_opaque_type(id, ty, ctx));
+            }
+        }
+
         let decl = {
             let decl = ty.declaration();
             decl.definition().unwrap_or(decl)
